/*!
 * SiTCP Remote Bus Control Protocol
 * Header file
 *
 * Copyright (C) 2016
 *		Bee Beans Technologies Co.,Ltd.
 *		All rights reserved.
 */
#ifndef RBCP_H
#define RBCP_H

#include <string>
#include <vector>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

class Rbcp
{
public:
    Rbcp(const std::string &address, int port, int retry = 3, int timeout = 3);
	virtual int Write(unsigned int address, const char *data, unsigned char length);
	virtual int Read(unsigned int address, char *data, unsigned char length);
	virtual void StrError(int sts, std::string &estr)
	{
		switch (sts) {
		case BusErr:
			estr += "Rbcp: Bus Error";
			break;
		case Timeout:
			estr += "Rbcp: Socket operation timed out";
			break;
		case Aborted:
			estr += "Rbcp: Aborted";
			break;
		case Mismatch:
			estr += "Rbcp: Mismatch";
			break;
		default:
			estr += "Rbcp: Unknown Error";
			break;
		}
	}

protected:
	struct rbcp_header {
		unsigned char type;
		unsigned char command;
		unsigned char id;
		unsigned char length;
		unsigned int address;
	};
	enum ErrSts {Timeout = -7, BusErr, SockErr, Mismatch, Aborted, Unknown};

	virtual int setReadCmd(unsigned int address, unsigned char length);
	virtual int setWriteCmd(unsigned int address, const char *data, unsigned char length);
	virtual int commRBCP(int recvSize, char id);
	virtual char getId()
	{
		m_id++;
		m_id = (m_id < 0)? 0 : m_id;
		return m_id;
	}
	char m_id;
	std::vector<char> m_buf;

	int m_retry;
	int m_sock;
	int m_timeout; // sec
	struct sockaddr_in m_sitcpAddr;

	static const unsigned char RBCP_VER	= 0xff;
	static const unsigned char RBCP_WR	= 0x80;
	static const unsigned char RBCP_RD	= 0xc0;
	static const int RBCP_HEADER_SIZE = sizeof(struct rbcp_header);
};

#endif // RBCP_H
