/*!
 * VmeGbeConfig class for VME-GbE2
 * Header file
 *
 * Copyright (C) 2016
 *		Bee Beans Technologies Co.,Ltd.
 *		All rights reserved.
 */
#ifndef VMEGBECONFIG_H
#define VMEGBECONFIG_H

#include <fstream>
#include <vector>

class Rbcp;

class VmeGbeConfig
{
public:
	VmeGbeConfig(const std::string &fileName);
	~VmeGbeConfig();
	virtual bool download(const std::string &address, int port, bool bEnabled = false);
	virtual bool setEventEnabled(unsigned char event);
	virtual bool clearEvent();
	virtual unsigned char getID();

protected:
	virtual bool good() { return m_fs.good(); }
	virtual void read(unsigned int &val);
	virtual void read(unsigned short &val);
	virtual void read(unsigned char &val);
	virtual void read(std::vector<char> &data);
	virtual void setRegister(unsigned int address, const char *data, unsigned char length);
	virtual void checkValid();
	virtual void printLog(const std::string &strLog);

	unsigned char m_eventMsk;
	unsigned char m_idMin; // for Interactive Mode
	unsigned char m_idMax; // for Interactive Mode
	unsigned char m_id; // for Interactive Mode
	unsigned int m_regEvent;
	std::string m_log;
	std::string m_filename;
	std::ifstream m_fs;
	Rbcp *m_pRbcp;

	const static unsigned short HEAD_INFO1 = 0x3330;
	const static unsigned short HEAD_USER1 = 0x3338;
	const static unsigned short HEAD_END = 0x333b;
	const static unsigned short HEAD_DOWNLOAD1 = 0x333c;
	const static unsigned short HEAD_DOWNLOAD2 = 0x333d;
	const static unsigned short HEAD_EVENT_ENABLE = 0x333e;
	const static unsigned short HEAD_DOWNLOAD = 0x333f;
	const static unsigned int NO_ADDR = 0xffffffff;
};

#endif // VMEGBECONFIG_H
